#include <sdcc-lib.h>
#include <pic16f648a.h>	// /usr/share/sdcc/include/pic/pic16f648a.h

#include "inline.h"
#include "libwait.h"

unsigned int __at 0x2007 __config =
	_CP_OFF &
	_DATA_CP_OFF &
	_LVP_OFF &
	_BOREN_OFF &	// for 3V
	_MCLRE_OFF &
	_PWRTE_OFF &
	_WDT_OFF &
	_INTRC_OSC_NOCLKOUT;

__code unsigned char __at 0x2100 eedata[] = {'d', 'u', 'm', 'm', 'y', 0 };

#define	LED		RA1
#define	DC_REL	RA0

/* Stepper Motor
			  _____
	X---3	 /	   \
		3	| Roter	|	XC, YC -> Vcc
	XC--3	|	o	|		  _  _
	_	3	|		|	X, Y, X, Y -> Collector / Drain
	X---3	 \_____/

			nnnnnnnnn
			|	|	|
			|	|	_
			Y	YC	Y
*/
/* 一相励磁(Wave drive)
		+---+			+---+			+---+		H:ON
X		|:	|			|:	|			|:	|
	----+:	+-----------+:	+-----------+:	+------	L:OFF
		 :				 :				 :
		 :	+---+		 :	+---+		 :	+---+	H:ON
Y		 :	|	|		 :	|	|		 :	|	|
	--------+	+-----------+	+-----------+	+-- L:OFF	0: 0000
		 :				 :				 :					a: 1000
_		 :		+---+	 :		+---+	 :		+--	H:ON	b: 0100
X		 :		|	|	 :		|	|	 :		|			c: 0010
	------------+	+-----------+	+-----------+ 	L:OFF	d: 0001
		 :				 :				 :					a: 1000
_		 :			+---+:			+---+:			H:ON	b: 0100
Y		 :			|	|:			|	|:					c: 0010
	----------------+	+-----------+	+---------- L:OFF	d: 0001
	 0   a   b   c   d   a   b   c   d   a   b   c
*/
/* 二相励磁(Full step drive)
		+-------+		+-------+		+-------+	H:ON
X		|:		|		|:		|		|:		|
	----+:		+-------+:		+-------+:		+--	L:OFF
		 :				 :				 :
		 :	+-------+	 :	+-------+	 :	+------	H:ON
Y		 :	|		|	 :	|		|	 :	|
	--------+		+-------+		+-------+		L:OFF	0: 0000
		 :				 :				 :					a: 1000
_		 :		+-------+:		+-------+:		+--	H:ON	b: 1100
X		 :		|		|:		|		|:		|			c: 0110
	------------+		+-------+		+-------+	L:OFF	d: 0011
		 :				 :				 :					a: 1001
_		 :			+-------+		+-------+		H:ON	b: 1100
Y		 :			|	 :	|		|	 :	|				c: 0110
	----------------+	 :	+-------+	 :	+------	L:OFF	d: 0011
	 0   a   b   c   d   a   b   c   d   a   b   c
*/
/* 一-二相励磁(Half stepping)
		+-----+			+-----+			+-----+		H:ON
X		|	  |			|	  |			|	  |
	----+	  +---------+	  +---------+	  +----	L:OFF
		:				:				:
		:	+-----+		:	+-----+		:	+-----+	H:ON
Y		:	|	  |		:	|	  |		:	|	  |
	--------+	  +---------+	  +---------+	  +	L:OFF	0: 0000
		:				:				:					a: 1000 -> 1000
_		:		+-----+	:		+-----+	:		+--	H:ON	b: 1100 -> 0100
X		:		|	  |	:		|	  |	|		|			c: 0110 -> 0010
	------------+	  +---------+	  +---------+	L:OFF	d: 0011 -> 0001
		:				:				:					a: 1001 -> 1000
_		:			+-----+			+-----+			H:ON	b: 1100 -> 0100
Y		:			|	: |			|	: |					c: 0110 -> 0010
	----------------+	: +---------+	: +--------	L:OFF	d: 0011 -> 0001
	 0   a . b . c . d . a . b . c . d . a . b . c
*/

void Intr(void) __interrupt 0 {
}

/*			  _______
	NC		-|2	 U	1|-	LED
	NC		-|3		0|-	DC_REL
	NC		-|4	 A	7|-	NC
	NC		-|5		6|-	NC
	GND		-|		 |-	Vcc
	MOTOR_X	-|0		7|-	NC
	MOTOR_Y	-|1		6|-	NC
	MOTOR_X'-|2	 B	5|-	NC
	MOTOR_Y'-|3_____4|-	NC
*/

void main(void) {

	unsigned char r;

				//  76543210
	PORTA		= 0b00000000;		// RA5: Input only, RA4: Open Drain
	CMCON		= 0b00000111;
	TRISA		= 0b00100000;		// 1: 入力, 0: 出力

	PORTB		= 0b00000000;
	TRISB		= 0b00000000;		// 1: 入力, 0: 出力

	OPTION_REG	= 0b11111111;		// 初期値:0b11111111
									// 7:PORTBプルアップ(0:使用する)
									// 6:RB0割り込みポート変化選択(0:立下り)
									// 5:TMR0クロックソース選択ビット(0:内部)
									// 4:TMR0加算タイミング(0:立上り)
									// 3:プリスケーラ割り当てビット(0:TMR0)
									// 2~0プリスケーラレート選択ビット(011:1/16)
									// 4MHz => 1MHz = 1,000,000 / 16 / 256 = 244Hz
									// 2~0プリスケーラレート選択ビット(100:1/32)

	INTCON		= 0b00000000;		// 初期値:0b0000000x
									// 7:GIE  Global Interrupt Enable bit
									// 6:PEIE Peripheral Interrupt Enable bit
									// 5:T0IE TMR0 Overflow Interrupt Enable bit
									// 4:INTE RB0/INT External Interrupt Enable bit
									// 3:RBIE RB Port Change Interrupt Enable bit
									// 2:T0IF TMR0 Overflow Interrupt Flag bit
									// 1:INTF RB0/INT External Interrupt Flag bit
									// 0:RBIF RB Port Change Interrupt Flag bit

	PIR1		= 0b00000000;		// 初期値:0b00000000(フラグ)
	PIE1		= 0b00000000;		// 初期値:0b00000000(イネーブル)
									// 7:EEIE EE Write Complete Interrupt Enable Bit
									// 6:CMIE Comparator Interrupt Enable bit
									// 5:RCIE USART Receive Interrupt Enable bit
									// 4:TXIE USART Transmit Interrupt Enable bit
									// 2:CCP1IE CCP1 Interrupt Enable bit
									// 1:TMR2IE TMR2 to PR2 Match Interrupt Enable bit
									// 0:TMR1IE TMR1 Overflow Interrupt Enable bit

	T2CON		= 0b00000000;		// 初期値:0b00000000
									// 6~3:TOUTPS Timer2 Output Postscale Select bits(0000:1/1, 0011:1/4, 1111:1/16)
									// 2:TMR2ON Timer2 On bit(1:ON)
									// 1~0:T2CKPS Timer2 Clock Prescale Select bits(00:1/1, 01:1/4, 1x:1/16)

//	forCalibOsc();

//------------------------------------------------------------------------------
//
//	main loop
//
	while(1) {

		waitL(10);								// 1000ms

		LED = 1;								// LED Off
		//
		//	Control Digital Cemera
		//
		//		http://blog.zaq.ne.jp/igarage/article/795/
		//
		// Half release
		DC_REL = 1;
		waitM(60);								// 30ms
		DC_REL = 0;
		waitM(200); waitM(200); waitM(200);		// 300ms

		// Full release
		DC_REL = 1;
		waitM(200);	waitM(100);					// 150ms
		DC_REL = 0;
		waitM(200); waitM(200);					// 200ms

		// Off release
		DC_REL = 1;
		waitM(60);								// 30ms
		DC_REL = 0;
		waitM(60);								// 30ms
		DC_REL = 1;
		waitM(60);								// 30ms
		DC_REL = 0;
		waitL(10);								// 1000ms

		LED = 0;								// LED On

		//
		//	Control Stepper Motor
		//
		for(r = 0; r < 90; r++) {

			PORTB = 0b00001001;					// Rotate 360 degree CW
			waitM(6);
			PORTB = 0b00001100;
			waitM(6);
			PORTB = 0b00000110;
			waitM(6);
			PORTB = 0b00000011;
			waitM(6);

//			PORTB = 0b00001001;					// Rotate 360 degree CCW
//			waitM(4);
//			PORTB = 0b00000011;
//			waitM(4);
//			PORTB = 0b00000110;
//			waitM(4);
//			PORTB = 0b00001100;
//			waitM(4);
		}

		PORTB = 0b00000000;
	}
}

